class FulfilmentMethod:
    """
    The fulfilment method. Fulfilled by the retailer (FBR) or fulfilled by
    bol.com (FBB).
    """

    FBR = "FBR"
    FBB = "FBB"


class TransporterCode:
    """
    https://api.bol.com/retailer/public/redoc/v3#tag/Transports
    """

    BPOST_BE = "BPOST_BE"
    BPOST_BRIEF = "BPOST_BRIEF"
    BRIEFPOST = "BRIEFPOST"
    COURIER = "COURIER"
    DHL = "DHL"
    DHLFORYOU = "DHLFORYOU"
    DHL_DE = "DHL_DE"
    DHL_GLOBAL_MAIL = "DHL-GLOBAL-MAIL"
    DPD_BE = "DPD-BE"
    DPD_NL = "DPD-NL"
    DYL = "DYL"
    FEDEX_BE = "FEDEX_BE"
    FEDEX_NL = "FEDEX_NL"
    FIEGE = "FIEGE"
    GLS = "GLS"
    LOGOIX = "LOGOIX"
    OTHER = "OTHER"
    PACKS = "PACKS"
    PARCEL_NL = "PARCEL-NL"
    RJP = "RJP"
    TNT = "TNT"
    TNT_BRIEF = "TNT_BRIEF"
    TNT_EXPRESS = "TNT-EXPRESS"
    TNT_EXTRA = "TNT-EXTRA"
    TRANSMISSION = "TRANSMISSION"
    TSN = "TSN"
    UPS = "UPS"


class CancellationReasonCode:
    OUT_OF_STOCK = "OUT_OF_STOCK"
    REQUESTED_BY_CUSTOMER = "REQUESTED_BY_CUSTOMER"
    BAD_CONDITION = "BAD_CONDITION"
    HIGHER_SHIPCOST = "HIGHER_SHIPCOST"
    INCORRECT_PRICE = "INCORRECT_PRICE"
    NOT_AVAIL_IN_TIME = "NOT_AVAIL_IN_TIME"
    NO_BOL_GUARANTEE = "NO_BOL_GUARANTEE"
    ORDERED_TWICE = "ORDERED_TWICE"
    RETAIN_ITEM = "RETAIN_ITEM"
    TECH_ISSUE = "TECH_ISSUE"
    UNFINDABLE_ITEM = "UNFINDABLE_ITEM"
    OTHER = "OTHER"



from decimal import Decimal
import dateutil.parser


class Field(object):
    def parse(self, api, xml, instance):
        raise NotImplementedError


class TextField(Field):
    def parse(self, api, xml, instance):
        return xml.text


class BooleanField(Field):
    def parse(self, api, xml, instance):
        return xml.text == "true"


class DecimalField(Field):
    def parse(self, api, xml, instance):
        return Decimal(xml.text)


class DateTimeField(Field):
    def parse(self, api, xml, instance):
        # WORKAROUND: At least on the test API I am bumping into this kind of
        # data: '2016-09-19+02:00'. Here, the time seems missing, only the
        # timezone offset is present. Let's detect this and handle gracefully.
        # Contacted BOL, awaiting reply...
        text = xml.text
        if text[10] == "+":
            text = text[:10] + "T00:00:00" + text[10:]
        # (end WORKAROUND)
        return dateutil.parser.parse(text)


class IntegerField(Field):
    def parse(self, api, xml, instance):
        return int(xml.text)


class ModelField(Field):
    def __init__(self, model):
        self.model = model

    def parse(self, api, xml, instance):
        return self.model.parse(api, xml)


class Model(object):
    @classmethod
    def parse(cls, api, xml):
        m = cls()
        m.xml = xml
        for element in xml:
            if "}" in element.tag:
                tag = element.tag.partition("}")[2]
            elif ":" in element.tag:
                tag = element.tag.partition(":")[2]
            else:
                tag = element.tag
            field = getattr(m.Meta, tag, TextField())
            setattr(m, tag, field.parse(api, element, m))
        return m


class ModelList(list, Model):
    @classmethod
    def parse(cls, api, xml):
        ml = cls()
        ml.xml = xml
        item_tag = getattr(ml.Meta, "item_type_tag", None)
        for element in xml:
            if item_tag and item_tag != element.tag:
                continue
            ml.append(ml.Meta.item_type.parse(api, element))
        return ml


class BillingDetails(Model):
    class Meta:
        pass


class ShipmentDetails(Model):
    class Meta:
        pass


class CustomerDetails(Model):
    class Meta:
        ShipmentDetails = ModelField(ShipmentDetails)
        BillingDetails = ModelField(BillingDetails)


class Invoice(Model):
    class Meta:
        pass


class InvoiceListItem(Model):
    class Meta:
        pass


class Invoices(ModelList):
    class Meta:
        item_type = InvoiceListItem
        item_type_tag = "InvoiceListItem"


class Price(Model):
    class Meta:
        PriceAmount = DecimalField()
        BaseQuantity = DecimalField()


class InvoiceSpecificationItem(Model):
    class Meta:
        Price = ModelField(Price)


class InvoiceSpecification(Model):
    class Meta:
        Item = ModelField(InvoiceSpecificationItem)
        pass


class InvoiceSpecifications(ModelList):
    class Meta:
        item_type = InvoiceSpecification


class OrderItem(Model):
    class Meta:
        OfferPrice = DecimalField()
        TransactionFee = DecimalField()
        Quantity = IntegerField()


class OrderItems(ModelList):
    class Meta:
        item_type = OrderItem


class Order(Model):
    class Meta:
        CustomerDetails = ModelField(CustomerDetails)
        OrderItems = ModelField(OrderItems)
        DateTimeCustomer = DateTimeField()
        DateTimeDropShipper = DateTimeField()


class Orders(ModelList):
    class Meta:
        item_type = Order


class ShipmentItem(Model):
    class Meta:
        OrderItem = ModelField(OrderItem)


class ShipmentItems(ModelList):
    class Meta:
        item_type = ShipmentItem


class Transport(Model):
    class Meta:
        pass


class Shipment(Model):
    class Meta:
        ShipmentDate = DateTimeField()
        ExpectedDeliveryDate = DateTimeField()
        ShipmentItems = ModelField(ShipmentItems)
        Transport = ModelField(Transport)


class Shipments(ModelList):
    class Meta:
        item_type = Shipment


class ProcessStatus(Model):
    class Meta:
        pass



VERSION = (0, 10, 0, "final", 0)

__title__ = "python-bol-api"
__version_info__ = VERSION
__version__ = ".".join(map(str, VERSION[:3])) + (
    "-{}{}".format(VERSION[3], VERSION[4] or "")
    if VERSION[3] != "final"
    else ""
)
__author__ = "Raymond Penners"
__license__ = "LGPL 3.0"
__copyright__ = "Copyright 2017-2022 Raymond Penners and contributors"



import json
from datetime import date
from decimal import Decimal

import dateutil.parser


def parse_json(content):
    return json.loads(content, parse_float=Decimal)


class Field(object):
    def parse(self, api, raw_data, instance):
        raise NotImplementedError


class RawField(Field):
    def parse(self, api, raw_data, instance):
        return raw_data


class DecimalField(Field):
    def parse(self, api, raw_data, instance):
        return Decimal(raw_data)


class DateTimeField(Field):
    def parse(self, api, raw_data, instance):
        return dateutil.parser.parse(raw_data)


class DateField(Field):
    def parse(self, api, raw_data, instance):
        parts = raw_data.split("-")
        if len(parts) != 3:
            raise ValueError(raw_data)
        iparts = list(map(int, parts))
        return date(*iparts)


class ModelField(Field):
    def __init__(self, model):
        self.model = model

    def parse(self, api, xml, instance):
        return self.model.parse(api, xml)


class BaseModel(object):
    @classmethod
    def parse(cls, api, content):
        m = cls()
        if isinstance(content, str):
            m.raw_content = content
            m.raw_data = parse_json(content)
        else:
            m.raw_content = None
            m.raw_data = content
        return m


class Model(BaseModel):
    @classmethod
    def parse(cls, api, content):
        m = super(Model, cls).parse(api, content)
        for tag, v in m.raw_data.items():
            field = getattr(m.Meta, tag, RawField())
            setattr(m, tag, field.parse(api, v, m))
        return m


class ModelList(list, BaseModel):
    @classmethod
    def parse(cls, api, content):
        ml = super(ModelList, cls).parse(api, content)
        items_key = getattr(ml.Meta, "items_key", None)
        if items_key:
            items = ml.raw_data.get(items_key)
        else:
            items = ml.raw_data
        if items:
            for item in items:
                ml.append(ml.Meta.item_type.parse(api, item))
        return ml


class BillingDetails(Model):
    class Meta:
        pass


class ShipmentDetails(Model):
    class Meta:
        pass


class CustomerDetails(Model):
    class Meta:
        shipmentDetails = ModelField(ShipmentDetails)
        billingDetails = ModelField(BillingDetails)


class Price(Model):
    class Meta:
        PriceAmount = DecimalField()
        BaseQuantity = DecimalField()


class OrderItem(Model):
    class Meta:
        offerPrice = DecimalField()
        transactionFee = DecimalField()
        latestDeliveryDate = DateField()
        expiryDate = DateField()


class OrderItems(ModelList):
    class Meta:
        item_type = OrderItem


class Order(Model):
    class Meta:
        customerDetails = ModelField(CustomerDetails)
        orderItems = ModelField(OrderItems)
        dateTimeOrderPlaced = DateTimeField()


class Orders(ModelList):
    class Meta:
        item_type = Order
        items_key = "orders"


class ShipmentItem(Model):
    class Meta:
        orderDate = DateTimeField()
        latestDeliveryDate = DateTimeField()


class ShipmentItems(ModelList):
    class Meta:
        item_type = ShipmentItem


class Transport(Model):
    class Meta:
        pass


class Shipment(Model):
    class Meta:
        shipmentDate = DateTimeField()
        shipmentItems = ModelField(ShipmentItems)
        transport = ModelField(Transport)


class Shipments(ModelList):
    class Meta:
        item_type = Shipment
        items_key = "shipments"


class ProcessStatus(Model):
    class Meta:
        createTimestamp = DateTimeField()


class ProcessStatuses(ModelList):
    class Meta:
        items_key = "processStatuses"
        item_type = ProcessStatus


class Invoice(Model):
    class Meta:
        pass


class Invoices(ModelList):
    class Meta:
        item_type = Invoice
        items_key = "invoiceListItems"


class InvoiceSpecificationItem(Model):
    class Meta:
        pass


class InvoiceSpecification(ModelList):
    class Meta:
        item_type = InvoiceSpecificationItem
        items_key = "invoiceSpecification"



import time
import requests
import hmac
import hashlib
import base64
from datetime import datetime, date
import collections
from enum import Enum

from xml.etree import ElementTree

from .models import (
    Orders,
    Shipments,
    ProcessStatus,
    Invoices,
    Invoice,
    InvoiceSpecifications,
)


__all__ = ["PlazaAPI"]


class TransporterCode(Enum):
    """
    https://developers.bol.com/documentatie/plaza-api/developer-guide-plaza-api/appendix-a-transporters/
    """

    DHLFORYOU = "DHLFORYOU"
    UPS = "UPS"
    TNT = "TNT"
    TNT_EXTRA = "TNT-EXTRA"
    TNT_BRIEF = "TNT_BRIEF"
    TNT_EXPRESS = "TNT-EXPRESS"
    COURIER = "COURIER"
    DYL = "DYL"
    DPD_NL = "DPD-NL"
    DPD_BE = "DPD-BE"
    BPOST_BE = "BPOST_BE"
    BPOST_BRIEF = "BPOST_BRIEF"
    BRIEFPOST = "BRIEFPOST"
    GLS = "GLS"
    FEDEX_NL = "FEDEX_NL"
    FEDEX_BE = "FEDEX_BE"
    OTHER = "OTHER"
    DHL = "DHL"
    DHL_DE = "DHL_DE"
    DHL_GLOBAL_MAIL = "DHL-GLOBAL-MAIL"
    TSN = "TSN"
    FIEGE = "FIEGE"
    TRANSMISSION = "TRANSMISSION"
    PARCEL_NL = "PARCEL-NL"
    LOGOIX = "LOGOIX"
    PACKS = "PACKS"
    RJP = "RJP"

    @classmethod
    def to_string(cls, transporter_code):
        if isinstance(transporter_code, TransporterCode):
            transporter_code = transporter_code.value
        assert transporter_code in map(
            lambda c: c.value, list(TransporterCode)
        )
        return transporter_code


class MethodGroup(object):
    def __init__(self, api, group):
        self.api = api
        self.group = group

    def request(self, method, path="", params={}, data=None):
        uri = path
        if not uri.startswith("/services"):
            uri = "/services/rest/{group}/{version}{path}".format(
                group=self.group, version=self.api.version, path=path
            )
        xml = self.api.request(method, uri, params=params, data=data)
        return xml

    def create_request_xml(self, root, **kwargs):
        elements = self._create_request_xml_elements(1, **kwargs)
        xml = """<?xml version="1.0" encoding="UTF-8"?>
<{root} xmlns="https://plazaapi.bol.com/services/xsd/v2/plazaapi.xsd">
{elements}
</{root}>
""".format(
            root=root, elements=elements
        )
        return xml

    def _create_request_xml_elements(self, indent, **kwargs):
        # sort to make output deterministic
        kwargs = collections.OrderedDict(sorted(kwargs.items()))
        xml = ""
        for tag, value in kwargs.items():
            if value is not None:
                prefix = " " * 4 * indent
                if isinstance(value, dict):
                    text = "\n{}\n{}".format(
                        self._create_request_xml_elements(indent + 1, **value),
                        prefix,
                    )
                elif isinstance(value, datetime):
                    text = value.isoformat()
                else:
                    text = str(value)
                # TODO: Escape! For now this will do I am only dealing
                # with track & trace codes and simplistic IDs...
                if xml:
                    xml += "\n"
                xml += prefix
                xml += "<{tag}>{text}</{tag}>".format(tag=tag, text=text)
        return xml


class OrderMethods(MethodGroup):
    def __init__(self, api):
        super(OrderMethods, self).__init__(api, "orders")

    def list(self):
        xml = self.request("GET")
        return Orders.parse(self.api, xml)


class InvoiceMethods(MethodGroup):
    def __init__(self, api):
        super(InvoiceMethods, self).__init__(api, "invoices")

    def list(self, order_id=None, period_from=None, period_to=None):
        params = {}
        if order_id:
            params["orderId"] = order_id
        if period_from or period_to:
            if not isinstance(period_from, date) or not isinstance(
                period_to, date
            ):
                raise ValueError()
            params["period"] = "/".join(
                [period_from.isoformat(), period_to.isoformat()]
            )
        xml = self.request("GET", "/services/rest/invoices", params=params)
        return Invoices.parse(self.api, xml)

    def get(self, invoice_id):
        xml = self.request(
            "GET", "/services/rest/invoices/{}".format(invoice_id)
        )
        return Invoice.parse(self.api, xml)

    def get_specification(self, invoice_id, page=None):
        params = {}
        if page is not None:
            params["page"] = page
        xml = self.request(
            "GET",
            "/services/rest/invoices/{}/specification".format(invoice_id),
            params=params,
        )
        return InvoiceSpecifications.parse(self.api, xml)


class ProcessStatusMethods(MethodGroup):
    def __init__(self, api):
        super(ProcessStatusMethods, self).__init__(api, "process-status")

    def get(self, id):
        xml = self.request("GET", "/{}".format(id))
        return ProcessStatus.parse(self.api, xml)


class ShipmentMethods(MethodGroup):
    def __init__(self, api):
        super(ShipmentMethods, self).__init__(api, "shipments")

    def list(self, page=None):
        if page is not None:
            params = {"page": page}
        else:
            params = None
        xml = self.request("GET", params=params)
        return Shipments.parse(self.api, xml)

    def create(
        self,
        order_item_id,
        date_time,
        expected_delivery_date,
        shipment_reference=None,
        transporter_code=None,
        track_and_trace=None,
    ):
        if transporter_code:
            transporter_code = TransporterCode.to_string(transporter_code)
        xml = self.create_request_xml(
            "ShipmentRequest",
            OrderItemId=order_item_id,
            DateTime=date_time,
            ShipmentReference=shipment_reference,
            ExpectedDeliveryDate=expected_delivery_date,
            Transport={
                "TransporterCode": transporter_code,
                "TrackAndTrace": track_and_trace,
            },
        )
        response = self.request("POST", data=xml)
        return ProcessStatus.parse(self.api, response)


class TransportMethods(MethodGroup):
    def __init__(self, api):
        super(TransportMethods, self).__init__(api, "transports")

    def update(self, id, transporter_code, track_and_trace):
        transporter_code = TransporterCode.to_string(transporter_code)
        xml = self.create_request_xml(
            "ChangeTransportRequest",
            TransporterCode=transporter_code,
            TrackAndTrace=track_and_trace,
        )
        response = self.request("PUT", "/{}".format(id), data=xml)
        return ProcessStatus.parse(self.api, response)


class PlazaAPI(object):
    def __init__(
        self, public_key, private_key, test=False, timeout=None, session=None
    ):
        self.public_key = public_key
        self.private_key = private_key
        self.url = "https://%splazaapi.bol.com" % ("test-" if test else "")
        self.version = "v2"
        self.timeout = timeout
        self.orders = OrderMethods(self)
        self.invoices = InvoiceMethods(self)
        self.shipments = ShipmentMethods(self)
        self.process_status = ProcessStatusMethods(self)
        self.transports = TransportMethods(self)
        self.session = session or requests.Session()

    def request(self, method, uri, params={}, data=None):
        content_type = "application/xml; charset=UTF-8"
        date = time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime())
        msg = """{method}

{content_type}
{date}
x-bol-date:{date}
{uri}""".format(
            content_type=content_type, date=date, method=method, uri=uri
        )
        h = hmac.new(
            self.private_key.encode("utf-8"),
            msg.encode("utf-8"),
            hashlib.sha256,
        )
        b64 = base64.b64encode(h.digest())

        signature = self.public_key.encode("utf-8") + b":" + b64

        headers = {
            "Content-Type": content_type,
            "X-BOL-Date": date,
            "X-BOL-Authorization": signature,
        }
        request_kwargs = {
            "method": method,
            "url": self.url + uri,
            "params": params,
            "headers": headers,
            "timeout": self.timeout,
        }
        if data:
            request_kwargs["data"] = data
        resp = self.session.request(**request_kwargs)
        resp.raise_for_status()
        tree = ElementTree.fromstring(resp.content)
        return tree



import requests


__all__ = ["OpenAPI"]


class MethodGroup(object):
    def __init__(self, api, group):
        self.api = api
        self.group = group

    def request(self, method, path, params={}):
        uri = "/{group}/{version}/{path}".format(
            group=self.group,
            path=path,
            version=self.api.version,
        )
        return self.api.request(method, uri, params=params)


class CatalogMethods(MethodGroup):
    def __init__(self, api):
        super(CatalogMethods, self).__init__(api, "catalog")

    def products(self, product_ids):
        path = "products/" + ",".join(product_ids)
        return self.request("GET", path)

    def search(self, query):
        """
        query might be 'Harry Potter', 'an_EAN' or 'an_ISBN'.
        For exact search, use extra quotation marks, for example:
        '"Harry Potter"'.
        """
        path = "search/"
        return self.request("GET", path, {"q": query})


class OpenAPI(object):
    def __init__(self, api_key, timeout=None, session=None):
        self.api_key = api_key
        self.url = "https://api.bol.com"
        self.version = "v4"
        self.catalog = CatalogMethods(self)
        self.timeout = timeout
        self.session = session or requests.Session()

    def request(self, method, uri, params={}):
        resp = self.session.get(
            self.url + uri,
            params=dict(params, **{"apikey": self.api_key}),
            timeout=self.timeout,
        )
        resp.raise_for_status()
        data = resp.json()
        return data



import sys

from setuptools import find_packages, setup

import bol


# Dynamically calculate the version based on bol.VERSION.
VERSION = bol.__version__

install_requires = ["python-dateutil", "requests"]

setup(
    name="python-bol-api",
    version=VERSION,
    description="Wrapper for the bol.com API",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Topic :: System",
        "Topic :: System :: Software Distribution",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: "
        "GNU Lesser General Public License v3 or later (LGPLv3+)",
    ],
    keywords="bol bol.com api wrapper",
    author="Raymond Penners",
    author_email="raymond.penners@intenct.nl",
    url="http://www.intenct.info/",
    license="",
    packages=find_packages(exclude=["ez_setup", "examples", "tests"]),
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    entry_points="",
)
